@@ -1,13 +1,15 @@
 #!/usr/bin/perl
-package FCGI::Daemon;
-our $VERSION = '0.20111121';
-use English '-no_match_vars';
-use FCGI 0.71;                          # on Debian available as libfcgi-perl
-use FCGI::ProcManager 0.18;             # on Debian available as libfcgi-procmanager-perl
-use 5.008;
 use strict;
 #use warnings;
 #use diagnostics -verbose;
+## no critic (TestingAndDebugging::RequireUseWarnings)
+package FCGI::Daemon;
+our $VERSION = '0.20151226';
+use 5.14.2;
+use English '-no_match_vars';
+use BSD::Resource;                      # on Debian available as libbsd-resource-perl
+use FCGI 0.71;                          # on Debian available as libfcgi-perl
+use FCGI::ProcManager 0.18;             # on Debian available as libfcgi-procmanager-perl
 use Getopt::Std;
 use autouse 'Pod::Usage'=>qw(pod2usage);
 
@@ -29,7 +31,19 @@ __PACKAGE__->run() unless caller();     # modulino i.e. executable rather than m
 =head2 help()
     print help screen extracted from POD
 =cut
-sub help { pod2usage(-verbose=>$ARG[0],-noperldoc=>1) and exit; }
+sub help { pod2usage(-verbose=>$ARG[0],-noperldoc=>1) and exit; }       ## no critic
+
+=head2 dieif()
+    exit handler
+=cut
+sub dieif {
+    if($ARG[0]){
+        my $err=$ARG[1];
+        unlink @o{'pidfile','sockfile'};
+        print "Error - $err:\n",$ARG[0],"\n";
+        exit 1;
+    }
+}
 
 =head2 run()
     Modulino-style main routine
@@ -53,29 +67,18 @@ sub run {
         $o{uid}=$o{u}||'www-data'; $o{uid_num}=scalar getpwnam($o{uid});
     }
 
-=head2 dieif()
-    exit handler
-=cut
-    sub dieif {
-        if($ARG[0]){
-            my $err=$ARG[1];
-            unlink @o{'pidfile','sockfile'};
-            print "Error - $err:\n",$ARG[0],"\n";
-            exit 1;
-        }
-    }
-
-    $SIG{INT}=$SIG{TERM}=sub{   # actually FCGI::ProcManager override our TERM handler so .sock and .pid files will be removed only by sysv script... :(
-                            $o{fcgi_pm}->pm_remove_pid_file() if $o{fcgi_pm};
-                            unlink @o{'sockfile','pidfile'};
-                            $o{fcgi_pm}->pm_die() if $o{fcgi_pm};   #pm_die() does not return
-                            exit 0;
-                         };
+    local $SIG{INT}= local $SIG{TERM}= sub{
+        # actually FCGI::ProcManager override our TERM handler so .sock and .pid files will be removed only by sysv script... :(
+        $o{fcgi_pm}->pm_remove_pid_file() if $o{fcgi_pm};
+        unlink @o{'sockfile','pidfile'};
+        $o{fcgi_pm}->pm_die() if $o{fcgi_pm};   #pm_die() does not return
+        exit 0;
+    };
 
     # daemonize
     if($o{'d'}){
         chdir '/';                              # this is good practice for unmounting
-        $PROGRAM_NAME='FCGI::Daemon';
+        local $PROGRAM_NAME='FCGI::Daemon';
         defined(my $pid=fork) or die "Can't fork: $!";
         exit if $pid;
         eval {use POSIX qw(setsid); POSIX::setsid();} or die q{Can't start a new session: }.$OS_ERROR;
@@ -105,19 +108,17 @@ sub run {
 
     # drop privileges if run as root
     if(defined $o{gid_num} and defined $o{uid_num}){
-        $REAL_GROUP_ID=$EFFECTIVE_GROUP_ID=getgrnam($o{gid});
+        local $REAL_GROUP_ID= local $EFFECTIVE_GROUP_ID= getgrnam($o{gid});
             dieif($OS_ERROR,'Unable to change group_id to '.$o{gid});
-        $REAL_USER_ID=$EFFECTIVE_USER_ID=getpwnam($o{uid});
+        local $REAL_USER_ID= local $EFFECTIVE_USER_ID= getpwnam($o{uid});
             dieif($OS_ERROR,'Unable to change user_id to '.$o{uid});
     }
 
     ## set rlimit(s)
-    require 'syscall.ph';
-    require 'sys/resource.ph';
-    my $rlim=pack 'L!L!',$o{rlimit_vmem},$o{rlimit_vmem};
-    dieif($OS_ERROR,"Unable to set RLIMIT_AS: ") if -1==syscall(SYS_setrlimit(),RLIMIT_AS(),$rlim);
-    $rlim=pack 'L!L!',$o{rlimit_cpu},$o{rlimit_cpu};
-    dieif($OS_ERROR,"Unable to set RLIMIT_CPU: ") if -1==syscall(SYS_setrlimit(),RLIMIT_CPU(),$rlim);
+    setrlimit(RLIMIT_AS, $o{rlimit_vmem}, $o{rlimit_vmem})
+      or warn "Unable to set RLIMIT_AS.\n";
+    setrlimit(RLIMIT_CPU, $o{rlimit_cpu}, $o{rlimit_cpu})
+      or warn "Unable to set RLIMIT_CPU.\n";
 
     REQ_LOOP:   # main loop
     while($rqst->Accept()>=0){
@@ -147,7 +148,7 @@ sub run {
             {
                 local *CORE::GLOBAL::exit=sub { die 'notr3a11yeXit' };
                 local $0=$req_env{SCRIPT_FILENAME};     #fixes FindBin (in English $0 means $PROGRAM_NAME)
-                no strict;                              # default for Perl5
+                no strict;                              ## no critic :: default for Perl5
                 do $0;                                  # do $0; could be enough for strict scripts
                 if($EVAL_ERROR){
                     $EVAL_ERROR=~s{\n+\z}{};
@@ -157,11 +158,11 @@ sub run {
 
             #untested experimental callback to execute on script exit
             #$_{$req_env{SCRIPT_FILENAME}}->{SIGTERM}() if defined $_{$req_env{SCRIPT_FILENAME}}->{SIGTERM};
-			#Perl scripts can cache persistent data in $_{$0}->{mydata}
-			#However if you store too much data it may trigger termination by rlimit
-			#After DO/EVAL $_{$0}->{'SIGTERM'} is being called so termination handler
-			#can be used to close DB connections etc.
-			#$_{$0}->{'SIGTERM'}=sub { print "I closed my handles"; };
+            #Perl scripts can cache persistent data in $_{$0}->{mydata}
+            #However if you store too much data it may trigger termination by rlimit
+            #After DO/EVAL $_{$0}->{'SIGTERM'} is being called so termination handler
+            #can be used to close DB connections etc.
+            #$_{$0}->{'SIGTERM'}=sub { print "I closed my handles"; };
 
             foreach(keys %main::){                      # cleanup garbage after do()
                 next if exists $allvars{$_};
@@ -219,7 +220,7 @@ sub run {
                 my $bytes=0;
                 while ($bytes<$req_env{'CONTENT_LENGTH'}){
                         $bytes+=read *STDIN,$buffer,($req_env{'CONTENT_LENGTH'}-$bytes);
-                        last if ($bytes==0 or !defined $bytes);
+                        last if ($bytes==0 or not defined $bytes);
                         print {$PIN} $buffer;
             }   }
             close $PIN;
@@ -239,15 +240,17 @@ sub run {
         }
         $o{fcgi_pm}->pm_post_dispatch();
     }
+    return;
 }
 
 # overriding process names
 sub FCGI::ProcManager::pm_change_process_name {
-        my ($self,$name)=@_;
-        my %p=( 'perl-fcgi-pm'  =>'FCGI::Daemon',
-                'perl-fcgi'     =>'FCGI::Daemon-worker',
-        );
-        $0=$p{$name} if $p{$name} ne '';
+    my ($self,$name)=@_;
+    my %p=( 'perl-fcgi-pm'  =>'FCGI::Daemon',
+            'perl-fcgi'     =>'FCGI::Daemon-worker',
+    );
+    $0=$p{$name} if $p{$name} ne '';                                    ## no critic
+    return;
 }
 
 =head2 get_file_from_path()
@@ -261,7 +264,7 @@ sub get_file_from_path {
         $file.='/'.$_;
         last if -f -s $file;
     }
-$file;
+    return $file;
 }
 
 1;
@@ -287,21 +290,27 @@ FCGI-Daemon run CGI scripts with RLIMITs and predefined number of workers.
 It was developed as replacement for cgiwrap-fcgi.pl (L<http://wiki.nginx.org/SimpleCGI>)
 and fcgiwrap (L<http://nginx.localdomain.pl/wiki/FcgiWrap>)
 
-Unlike fcgiwrap, FCGI-Daemon correctly passing STDERR to web server.
-
 FCGI-Daemon check for executable in path and correctly set PATH_INFO
-environment variable which is crucial for some CGI applications like 
+environment variable which is crucial for some CGI applications like
 fossil (L<http://fossil-scm.org>). 
 (Lack of this functionality make cgiwrap-fcgi.pl unsuitable for some scripts.)
 
 
 =head1 FEATURES
 
-* drop privileges when run as root
-* setrlimit for RLIMIT_AS and RLIMIT_CPU
-* detection of script executable in path (PATH_INFO)
-* DOing .pl - run CGI scripts in Perl with persistent interpreter (like mod_perl).
-* detection of memory leaks
+=over 4
+
+=item * drop privileges when run as root
+
+=item * setrlimit for RLIMIT_AS and RLIMIT_CPU
+
+=item * detection of script executable in path (PATH_INFO)
+
+=item * DOing .pl - run CGI scripts in Perl with persistent interpreter (like mod_perl).
+
+=item * detection of memory leaks
+
+=back
 
 =cut
 
@@ -310,7 +319,7 @@ fossil (L<http://fossil-scm.org>).
 It can be manually invoked as "perl /usr/share/perl5/FCGI/Daemon.pm"
 or with included SysV init script.
 
-Please make sure that user should have write permissions for sock file.
+Sock file should be write-able.
 
 =head1 OPTIONS
 
@@ -342,7 +351,7 @@ All options are optional.
 By default FCGI::Daemon DOing .pl scripts up to B<-e> times.
 This is several times faster than invoking Perl for every call of CGI script.
 This option define how often parent process (worker) should restart.
-Warning - some scripts may be incompatible with this so disable with "-e0" if necessary.
+Warning: some scripts may be incompatible with this so disable with "-e0" if necessary.
 
 =item B<-f>
 
@@ -350,9 +359,9 @@ By default only .pl scripts executed by persistent interpreter.
 However some Perl scripts may have .cgi extension
 so to accelerate such scripts a following regex may be used:
 I<perlcgi/[^/]+\.cgi|\.pl>
-Where I<perlcgi> stand for path so not all .cgi will be treated as perl but
+Where I<perlcgi> stands for path so not all .cgi will be treated as perl but
 only ones from I<perlcgi> folder (or from folder which name ends with perlcgi).
-This regex will be anchored to end of file name.
+This regex is anchored to end of file name.
 
 
 =item B<-l>
@@ -369,9 +378,10 @@ This is helpful for scripts that are leaking memory
 FCGI
 FCGI::ProcManager
 
-For Debian GNU/Linux (perhaps the best platform)
-required moduled provided by the following packages:
+For Debian GNU/Linux (recommended platform)
+required modules provided by the following packages:
 
+libbsd-resource-perl
 libfcgi-perl
 libfcgi-procmanager-perl
 
@@ -403,7 +413,7 @@ B<Why not fcgiwrap?>
 
 =over
 
-=item fcgiwrap doesn't pass STDERR to web server for logging.
+=item fcgiwrap do not have special support for Perl scripts.
 
 =back
 
@@ -420,7 +430,7 @@ Frankly, it is not very beautiful.
 
 =item - It takes no options so you have to modify the code.
 
-=item - It is incompatible with some CGI applications, notably with fossil due to lack of support for PATH_INFO
+=item - It is incompatible with some CGI applications, notably with fossil due to lack of support for PATH_INFO.
 
 =back
 
@@ -436,7 +446,7 @@ During development of this module a bug in Perl was discovered:
 
 =head1 SUPPORT
 
-After installing, you can find documentation for this module with the perldoc command.
+After installing, you can read documentation for this module with the perldoc command.
 
     perldoc FCGI::Daemon
 
@@ -465,7 +475,7 @@ Dmitry Smirnov, C<< <onlyjob at cpan.org> >>
 =head1 LICENSE
 
 FCGI::Daemon - FastCGI daemon
-Copyright (C) 2011 Free Software Foundation
+Copyright (C) 2011,2015 Free Software Foundation
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
